{
(* $Id: slex.mll 858 2008-02-10 22:21:11Z isr $ *)
(*
 * This file is copied from oscm (Scheme Interpreter).
 * http://fxp.hp.infoseek.co.jp/ocaml/oscm/
 *)
  open Sparser
}

let ws = [' ' '\t' '\n' '\r']
let dig = ['0'-'9']
let alp = ['a'-'z' 'A'-'Z']
let head = ['!' '$' '%' '&' '*' '/' ':' '<' '=' '>' '?' '^' '_' '~']
let tail = ['+' '-' '.' '@']

rule scheme = parse
  ws+ { scheme lexbuf }
| ";" [^'\n']* "\n"  { scheme lexbuf }

| "#(" { SPAREN }
| "("  { LPAREN }
| ")"  { RPAREN }
| "'"  { QUOTE }
| "`"  { QUASIQUOTE }
| ","  { UNQUOTE }
| ",@" { UNQUOTESPLICE }
| "."  { PERIOD }

| (alp|head) (alp|head|dig|tail)*
| "+" | "-" | "..."
  { SYMBOL  (Lexing.lexeme lexbuf) }

| "#t" | "#T" { BOOL true }
| "#f" | "#F" { BOOL false }

| "#\\" ['s' 'S']['p' 'P']['a' 'A']['c' 'C']['e' 'E'] { CHAR ' ' }
| "#\\" ['n' 'N']['e' 'E']['w' 'W']['l' 'L']['i' 'I']['n' 'N']['e' 'E'] { CHAR '\n' }
| "#\\" _ { CHAR (Lexing.lexeme lexbuf).[2] }

| "\""
  { let rec iter cur = match (char_elem lexbuf) with
        None -> cur
      | Some c -> iter (c::cur) in
    let base = iter [] in
    let str = String.create (List.length base) in
    let _ = List.fold_right (fun c i -> str.[i]<-c;i+1) base 0 in
    STRING str
  }

| dig+ as i { INT (Big_int.big_int_of_string i) }
| dig+ '.' dig* as f { FLOAT (float_of_string f) }
(*
 * | ("#i" | "#e" | "#b" | "#o" | "#d" | "#x" |
 *    "#I" | "#E" | "#B" | "#O" | "#D" | "#X" | "+" | "-" | "." | dig)
 *     [^' ' '\n' '\t' '(' ')' '\"' ';']*
 *   { NUMBER (Numeric.of_string (Lexing.lexeme lexbuf) 10) }
 *)
(* | ['+' '-']? dig+ { NUMBER (int_of_string (Lexing.lexeme lexbuf)) } *)

| eof { raise End_of_file }


and char_elem = parse
    '\"' { None }
  | "\\\\" { Some '\\' }
  | "\\\"" { Some '\"' }
  | "\\n" | "\\N" { Some '\n' }
  | _ { Some (Lexing.lexeme lexbuf).[0] }
