{
(* $Id: tlex.mll 879 2008-02-12 07:05:52Z isr $ *)
  open Tparser
  exception Eof

  type token = Tparser.token

  let keyword_table =
      Utils.create_hashtable 32 [
        ("and",     AND);
        ("break",   BREAK);
        ("curry",   CURRY);
        ("def",     DEF);
        ("defined", DEFINED);
        ("do",      DO);
        ("each",    EACH);
        ("eh",      EACH);
        ("enum",    ENUM);
        ("if",      IF);
        ("lambda",  LAMBDA);
        ("loop",    LOOP);
        ("nameof",  NAMEOF);
        ("next",    NEXT);
        ("or",      OR);
        ("print",   PRINT);
        ("return",  RETURN);
        ("struct",  STRUCT);
        ("then",    THEN);
        ("times",   TIMES);
        ("veach",   VEACH);
        ("veh",     VEACH);
        ("while",   WHILE)
      ]
}

let digit = ['0'-'9']
let octit = ['0'-'7']
let hexit = ['0'-'9' 'A'-'F' 'a'-'f']
let id = ['a'-'z' 'A'-'Z' '_']['a'-'z' 'A'-'Z' '_' '0'-'9']*

let not_line  = [^ '\n' ]
let whitechar = [' ' '\t' '\n' '\r']

rule tulpeta = parse
    "/*"        { comment lexbuf }
  | '#'         { line_comment lexbuf }
  | "->"        { MNGT }
  | '\\'        { BKSL }
  | ','         { CM }
  | ':'         { CL }
  | '.'         { DOT }
  | ';'         { SM }
  | ";;"        { SMSM }
  | '('         { LP }
  | ')'         { RP }
  | '['         { LB }
  | ']'         { RB }
  | '{'         { LC }
  | '}'         { RC }
  | "!="        { NOEQ }
  | "=="        { EQEQ }
  | '<'         { LTHAN }
  | "<<"        { LTLT }
  | "><"        { VAR "><" }
  | '='         { ASSIGN }
  | "+="        { ADD_ASSIGN }
  | '+'         { ADD }
  | '-'         { MINUS }
  | '*'         { ASTR }
  | '/'         { DIV }
  | '%'         { MOD }
  | id as s     {
      try
          Hashtbl.find keyword_table s
      with Not_found ->
          VAR s
  }
  | digit+ as i { INT (Big_int.big_int_of_string i) }
  | digit+ '.' digit* as f { FLOAT (float_of_string f) }
  | '"'         { STRING (quoted_str lexbuf) }
  | _           { tulpeta lexbuf }
  | "=====START_LANG(" id ")=====" not_line {
      print_endline "error: Line feed isn't put after START_LANG";
      assert false
  }
  | "=====START_LANG(" (id as lname) ")=====" '\n' {
    CHANGE_LANG (lname, (lang_src lexbuf))
  }
  | eof         {
      lexbuf.Lexing.lex_eof_reached <- true;
      EOF
  }

and comment = parse
    "*/"        { tulpeta lexbuf }
  | _           { comment lexbuf }

and line_comment = parse
    '\n'        { tulpeta lexbuf }
  | _           { line_comment lexbuf }

and quoted_str = parse
    "\\\"" as q { q ^ (quoted_str lexbuf) }
  | '"'         { "" }
  | _           { let cur = Lexing.lexeme lexbuf in cur ^ (quoted_str lexbuf) }

and lang_src = parse
    "=====END_LANG=====" { "" }
  | _           { let cur = Lexing.lexeme lexbuf in cur ^ (lang_src lexbuf) }
  
