%{
(* $Id: tparser.mly 871 2008-02-12 04:16:53Z isr $ *)
open Utils
open Ttype
open Texec

let makeVarNode v = initExp N_VAR (Some v) None None

%}

%token <Big_int.big_int> INT
%token <float>  FLOAT
%token <string> STRING VAR
%token <string * string> CHANGE_LANG
%token ADD
%token ADD_ASSIGN
%token AND
%token ASSIGN
%token ASTR
%token BKSL
%token BREAK
%token CL
%token CM
%token CURRY
%token DEF
%token DEFINED
%token DIV
%token DO
%token DOT
%token EACH
%token ENUM
%token EQEQ
%token EOF
%token IF
%token LAMBDA
%token LB
%token LC
%token LOOP
%token LP
%token LTHAN
%token LTLT
%token MINUS
%token MNGT
%token MOD
%token NAMEOF
%token NEXT
%token NOEQ
%token OR
%token PRINT
%token RB
%token RC
%token RETURN
%token RP
%token SM
%token SMSM
%token STRUCT
%token THEN
%token TIMES
%token VEACH
%token WHILE

%start input
%type <Ttype.stmt list> input

%%

input:
    EOF /* empty */ { [] }
  | stmts EOF       { $1 }
  | SMSM            { [] }
  | stmts SMSM      { $1 }
  | exp EOF         { [initStmt STMT_CALC None None (Some $1) None] }

stmts:
    stmts stmt { $1 @ [$2] }
  | stmt       { [$1] }

stmt:
    stmt_change_lang { $1 }
  | stmt_compound    { $1 }
  | stmt_empty       { $1 }
  | stmt_exp         { $1 }
  | stmt_iteration   { $1 }
  | stmt_jump        { $1 }
  | stmt_select      { $1 }
  | stmt_print       { $1 }
  | stmt_funcdef     { $1 }
  | stmt_struct      { $1 }

stmt_struct:
    STRUCT VAR LC struct_dec_list RC { initStmtStruct (Some (initStData $2 $4)) } 

struct_dec_list:
    struct_dec_list struct_dec { $1 @ [$2] }
  | struct_dec { [$1] }

struct_dec:
    struct_data  { StVar   $1 }
  | struct_label { StLabel $1 }
  | stmt_funcdef { StDef   $1 }
  | stmt_initial { $1 }
  | stmt_easy_funcdef { StDef $1 }
  | stmt_struct { StData (fromJust $1.sStruct) }

struct_label:
    VAR CL { $1 }
  | DEF CL { "def" }

struct_data:
    VAR SM { $1 }

stmt_initial:
    VAR ASSIGN exp SM { StInit ($1, $3) }

stmt_funcdef:
    DEF VAR LP RP stmt_compound {
      initStmt STMT_DEF None (Some $5) (Some (makeVarNode $2)) None
  }
  | DEF VAR LP exp_vars RP stmt_compound { 
      initStmt STMT_DEF (Some $4) (Some $6) (Some (makeVarNode $2)) None
  }

stmt_easy_funcdef:
    VAR LP RP stmt_compound {
      initStmt STMT_DEF None (Some $4) (Some (makeVarNode $1)) None
  }
  | VAR LP exp_vars RP stmt_compound {
      initStmt STMT_DEF (Some $3) (Some $5) (Some (makeVarNode $1)) None
  }
 
exps:
    exps CM exp { initExp N_CM_LIST None (Some $1) (Some $3) }
  | exp         { $1 }

exp_array:
    LB exps RB { initExp N_ARRAY None None (Some $2) }
  | LB RB      { initExp N_ARRAY None None None }

exp_lambda:
    LC exp RC                    { initExp N_LAMBDA (Some "{}") None (Some $2) }
  | LC exp_vars MNGT exp RC      { initExp N_LAMBDA (Some "{}") (Some $2) (Some $4) }
  | LP BKSL exp RP               { initExp N_LAMBDA (Some "(\\)") None (Some $3) }
  | LP BKSL exp_vars MNGT exp RP { initExp N_LAMBDA (Some "(\\)") (Some $3) (Some $5) }
  | LAMBDA LP exp RP { initExp N_LAMBDA (Some "lambda()") None (Some $3) }
  | LAMBDA LP exp_vars MNGT exp RP {
      initExp N_LAMBDA (Some "lambda()") (Some $3) (Some $5)
  }

stmt_compound:
    LC stmts RC { initStmt STMT_COMPOUND None None None (Some $2) }

stmt_select:
    IF exp THEN stmt { initStmt STMT_IF (Some $2) (Some $4) None None }

stmt_iteration:
    EACH VAR exp DO stmt { 
      initStmt STMT_EACH  (Some $3) (Some $5) (Some (makeVarNode $2)) None
  }
  | EACH VAR CL exp DO stmt {
      initStmt STMT_EACH  (Some $4) (Some $6) (Some (makeVarNode $2)) None
  }
  | VEACH VAR exps DO stmt {
      initStmt STMT_VEACH (Some $3) (Some $5) (Some (makeVarNode $2)) None
  }
  | VEACH VAR CL exps DO stmt {
      initStmt STMT_VEACH (Some $4) (Some $6) (Some (makeVarNode $2)) None
  }
  | TIMES exp DO stmt    { initStmt STMT_TIMES   (Some $2) (Some $4) None None }
  | LOOP stmt            { initStmt STMT_LOOP    None      (Some $2) None None }
  | WHILE exp DO stmt    { initStmt STMT_WHILE   (Some $2) (Some $4) None None }
  | DO stmt WHILE exp SM { initStmt STMT_DOWHILE (Some $4) (Some $2) None None }

stmt_jump:
    RETURN SM     { initStmt STMT_RETURN None None None None }
  | RETURN exp SM { initStmt STMT_RETURN None None (Some $2) None }
  | BREAK SM      { initStmt STMT_BREAK None None None None }
  | NEXT SM       { initStmt STMT_NEXT None None None None }

stmt_empty: SM { initStmt STMT_EMPTY None None None None }

stmt_exp: exp SM { initStmt STMT_EXP None None (Some $1) None }

stmt_change_lang:
  CHANGE_LANG {
    let lname = initExp N_STRING (Some (fst $1)) None None in
    let src = initExp N_STRING (Some (snd $1)) None None in
    initStmt STMT_CHANGE_LANG (Some lname) None (Some src) None
  }

stmt_print:
    PRINT stmt_exp     { initStmt STMT_PRINT None (Some $2) None None }
  | PRINT stmt_empty   { initStmt STMT_PRINT None (Some $2) None None }

exp:
    exp_andor IF exp { initExp N_IF None (Some $1) (Some $3) }
  | exp_andor { $1 }

exp_andor:
    exp_andor AND exp_assign { initExp N_AND None (Some $1) (Some $3) } 
  | exp_andor OR  exp_assign { initExp N_OR  None (Some $1) (Some $3) } 
  | exp_assign { $1 }

exp_assign:
    exp_postfix ASSIGN     exp_assign { initExp N_ASSIGN     None (Some $1) (Some $3) }
  | exp_postfix ADD_ASSIGN exp_assign { initExp N_ADD_ASSIGN None (Some $1) (Some $3) }
  | exp_postfix LTLT exp_assign { initExp N_LTLT None (Some $1) (Some $3) }
  | exp_equality { $1 }

exp_equality:
    exp_equality EQEQ exp_relational { initExp N_EQEQ None (Some $1) (Some $3) }
  | exp_equality NOEQ exp_relational { initExp N_NOEQ None (Some $1) (Some $3) }
  | exp_relational { $1 }

exp_relational:
    exp_relational LTHAN exp_addsub { initExp N_LTHAN None (Some $1) (Some $3) }
  | exp_addsub { $1 }

exp_addsub:
    exp_addsub ADD  exp_multi  { initExp N_ADD  None (Some $1) (Some $3) }
  | exp_addsub MINUS exp_multi { initExp N_MINUS None (Some $1) (Some $3) }
  | exp_multi { $1 }

exp_multi:
    exp_multi ASTR  exp_unary { initExp N_TIMES None (Some $1) (Some $3) }
  | exp_multi DIV   exp_unary { initExp N_DIV   None (Some $1) (Some $3) }
  | exp_multi MOD   exp_unary { initExp N_MOD   None (Some $1) (Some $3) }
  | exp_unary { $1 }

exp_unary:
    MINUS   exp_unary { initExp N_UMINUS  None None (Some $2) }
  | NAMEOF  exp_unary { initExp N_NAMEOF  None None (Some $2) }
  | DEFINED exp_unary { initExp N_DEFINED None None (Some $2) }
  | exp_postfix { $1 }

exp_postfix:
    exp_postfix LB exp RB  { initExp N_ARRAY_R   None (Some $1) (Some $3) }
  | exp_postfix LP RP      { initExp N_CALL_FUNC None (Some $1) None }
  | exp_postfix LP exps RP { initExp N_CALL_FUNC None (Some $1) (Some $3) }
  | CURRY LP exps RP { initExp N_CURRY None None (Some $3) }
  | exp_postfix DOT VAR {
      initExp N_DOT None (Some $1) (Some (makeVarNode $3))
  }
  | factor { $1 }

factor:
    INT    { initExp N_INT    (Some (Big_int.string_of_big_int $1)) None None }
  | FLOAT  { initExp N_FLOAT  (Some (string_of_float $1)) None None }
  | STRING { initExp N_STRING (Some $1) None None }
  | VAR    { initExp N_VAR    (Some $1) None None }
  | LP exp RP  { initExp N_PRIMARY None None (Some $2) }
  | exp_lambda { $1 }
  | exp_array  { $1 }

exp_vars:
    exp_vars CM exp_var { initExp N_CM_DEF_ARG None (Some $1) (Some $3) }
  | exp_var { $1 }

exp_var:
    ASTR exp_var_factor { initExp N_V_ARGS None None (Some $2) }
  | exp_var_factor { $1 }

exp_var_factor: VAR { initExp N_VAR (Some $1) None None }

%%

