(* $Id: ttype.mli 871 2008-02-12 04:16:53Z isr $ *)
val cancel_name : string
type t_node 
    = N_ADD
    | N_ADD_ASSIGN
    | N_AND
    | N_ARRAY
    | N_ARRAY_R     (* is N_ARRAY_REFERENCE *)
    | N_ASSIGN
    | N_CALL_FUNC
    | N_CANCEL
    | N_CM_DEF_ARG
    | N_CM_LIST
    | N_CURRY
    | N_DEFINED
    | N_DIV
    | N_DOT
    | N_EQEQ
    | N_FLOAT
    | N_IF
    | N_INT
    | N_LAMBDA
    | N_LTHAN
    | N_LTLT
    | N_MINUS
    | N_MOD
    | N_NAMEOF
    | N_NOEQ
    | N_OR
    | N_PRIMARY
    | N_STRING
    | N_TIMES
    | N_UMINUS
    | N_V_ARGS
    | N_VAR
type exp = {
  eType : t_node;
  eLabel : string option;
  eLeft : exp option;
  eRight : exp option;
}
val initExp : t_node -> string option -> exp option -> exp option -> exp
val eGetLabel : exp -> string
val eGetLeft : exp -> exp
val eGetRight : exp -> exp
val getStringsFromVarNodes : exp option -> string list
val getArgNodeNum : exp -> int
val cutExpNodes : exp option -> int list -> exp option
type t_stmt
  = STMT_BREAK
  | STMT_CALC
  | STMT_CHANGE_LANG
  | STMT_COMPOUND
  | STMT_DEF
  | STMT_DOWHILE
  | STMT_EACH
  | STMT_EMPTY
  | STMT_EXP
  | STMT_IF
  | STMT_LOOP
  | STMT_NEXT
  | STMT_PRINT
  | STMT_RETURN
  | STMT_STRUCT
  | STMT_TIMES
  | STMT_VEACH
  | STMT_WHILE
type stmt = {
  sType : t_stmt;
  sCond : exp option;
  sNest : stmt option;
  sExp : exp option;
  sComp : stmt list option;
  sStruct : stData option;
}
and stData = {
  stName : string;
  stDecls : stDecl list;
}
and stDecl =
    StData of stData
  | StVar of string
  | StDef of stmt
  | StLabel of string
  | StInit of string * exp

val initStmt :
  t_stmt ->
  exp option -> stmt option -> exp option -> stmt list option -> stmt
val initStmtStruct : stData option -> stmt
val sGetCond : stmt -> exp
val sGetNest : stmt -> stmt
val sGetExp : stmt -> exp
val sGetComp : stmt -> stmt list
val sGetStruct : stmt -> stData
val initStData : string -> stDecl list -> stData
type labelValue = string * varValue
and tyStruct = {
  tyName : string;
  mutable tyInits : labelValue list;
  mutable tyVars   : string list;
  mutable tyDefs   : stmt list;
  mutable tyTySt   : tyStruct list;
}
(* instance object *)
and insObj = {
  inType : tyStruct;
  inVars : labelValue list;
}

and varTbl = (string, varDef) Hashtbl.t
and varManager = varTbl list
and vstmt = stmt * exp option * varDef list * varManager

and varValue =
    VArray of varValue list
  | VCncl   (* cancel *)
  | VFloat of float
  | VIns   of insObj (* instance *)
  | VInt   of Big_int.big_int
    (* stmt is function, and exp is the argument list. *)
  | VStmt of vstmt
  | VStr   of string
  | VTySt  of tyStruct
and varDef = { vdType : eVarType; vdLabel : string; vdValue : varValue; }
and eVarType =
    VD_ARRAY
  | VD_CNCL
  | VD_FLOAT
  | VD_FUNC
  | VD_INS
  | VD_INT
  | VD_STR
  | VD_TYST

val showVal : varValue -> string
val printVal : varValue -> string
val getVarInt : varValue -> Big_int.big_int
val getVarFloat : varValue -> float
val getVarArray : varValue -> varValue list
val getVarStmt : varValue -> vstmt
val getVarString : varValue -> string
val gvmi : varValue -> int
val gvi : varValue -> Big_int.big_int
val gvf : varValue -> float
val gva : varValue -> varValue list
val gvs : varValue -> string
val is_vfloat : varValue -> bool
val is_one_vfloat : varValue -> varValue -> bool
val iof : varValue -> varValue -> bool
val getVType : varValue -> eVarType

val vzero : varValue
val vunit : varValue

val initVarDef : eVarType -> string -> varValue -> varDef
type retExpCalc = varValue
val getExpValue : retExpCalc -> varValue
val initVarDefWithExp : exp -> varValue -> varDef
val initVarDefWithName : string -> varValue -> varDef
val initDefFunc : varManager -> stmt -> varDef
val getVArgs : varValue list -> varValue
val getMember : varManager -> varValue -> string -> varValue
val curryVStmt : varValue -> varValue list -> varValue
val getStringFromTree : exp -> string
