(* $Id: vars.ml 844 2008-02-08 19:17:59Z isr $ *)

open Ttype

let initVarsMng : varManager =
  [Hashtbl.create 1000]

(* NOTE: decVarsMng does NOT decrease when nest 0. *)
let incVarsMng mng : varManager = Hashtbl.create 100 :: mng
let topVarsMng  mng : varTbl = List.hd mng
let pushVarsMng mng tbl : varManager = tbl::mng
let rec decVarsMng mng (n: int) : varManager =
  assert (n <= List.length mng);
  if List.length mng = n then mng else decVarsMng (List.tl mng) n

let getCurNestNum mng : int = List.length mng

(*
 * Get a value from the table.
 *)
let findVarDefTbl tbl (nm: string) : varDef option =
  try
    Some (Hashtbl.find tbl nm)
  with Not_found ->
    None

let rec findVarDef mng (nm: string) : varDef =
  match mng with
    [] -> prerr_endline ("error: \"" ^ nm ^ "\" isn't defined");
          assert false
  | tbl::rs -> let ret = findVarDefTbl tbl nm in 
               if ret <> None then Utils.fromJust ret
               else findVarDef rs nm
  
let findVarDefWithExp mng (e: exp) : varDef =
  findVarDef mng (eGetLabel e)
let findVDexp = findVarDefWithExp

(* 
 * Assign operation.
 *)
let asgnVarDefTbl tbl (v: varDef) : unit =
  Hashtbl.replace tbl v.vdLabel v

let asgnVarDefListTbl tbl (vs: varDef list) : unit =
  List.iter (fun v -> asgnVarDefTbl tbl v) vs

let rec asgnVarDef mng (v: varDef) : unit =
  asgnVarDefTbl (List.hd mng) v
 
let asgnVarDefList mng (vs: varDef list) : unit =
  asgnVarDefListTbl (List.hd mng) vs

(* Is there? *)
let existVarDefTbl tbl (nm: string) : bool =
  Hashtbl.mem tbl nm
let rec existVarDefWithExp mng (e: exp) : bool =
  match mng with
    [] -> false
  | tbl::rs -> if existVarDefTbl tbl (eGetLabel e)
               then true
               else existVarDefWithExp rs e

let existVDexp = existVarDefWithExp
